import os
import inspect


class DEBUGR:

    @classmethod
    def new(cls, base_depth=0):
        if 'DEBUG' in os.environ:
            return DEBUGR(base_depth)
        return NDEBUGR(base_depth)

    def __init__(self, base_depth=0):
        self.base_depth = base_depth

    def reset_depth(self):
        if self.base_depth:
            return
        depth = len(inspect.stack())
        self.base_depth = depth

    def MSG(self, obj, msg, pfx='  '):
        try:
            cref = obj.__class__.__name__ + '.'
        except AttributeError:
            cref = ''
        caller_fn = inspect.stack()[1].function
        depth = len(inspect.stack())
        depth_pfx = pfx * (depth - self.base_depth)
        prefix = f'{depth_pfx} {cref}{caller_fn}():'
        print(f"DEBUG:{prefix}{msg}")

    def OBJ(self, po, name, obj, pfx='  ', ev='->'):
        self.MSG(po, f' {ev} {name}={obj}', pfx=pfx)
        if not hasattr(obj, 'OX'):
            return
        pd = ' ' * len(ev)
        for line in obj.OX:
            self.MSG(po, f' {pd} {name}{line}', pfx=pfx)

    def VAL(self, obj, name, val, pfx='  '):
        try:
            cref = obj.__class__.__name__ + '.'
        except AttributeError:
            cref = ''
        caller_fn = inspect.stack()[1].function
        depth = len(inspect.stack())
        depth_pfx = pfx * (depth - self.base_depth)
        prefix = f'{depth_pfx} {cref}{caller_fn}():'
        print(f"DEBUG:{prefix}{name}={val!r}")

    def M_M(self, obj, r, pfx='  '):
        self.MSG(obj, f'match! result={r}', pfx=' .')
#       self.MSG(obj, f'       result.rest={r.rest}', pfx=' .')
#       self.MSG(obj, f'       result.params={r.params}', pfx=' .')


class NDEBUGR(DEBUGR):

    def reset_depth(self):
        return

    def VAL(self, obj, name, val, pfx='  '):
        return

    def MSG(self, obj, msg, pfx='  '):
        return

    def M_M(self, obj, r, pfx='  '):
        return


DBG = DEBUGR.new()
